% function values = calc_sens_spec(cc, flag_mean=0)
%
% Calculates sensitivities and specificities for each class (row of cc)
%
% Inputs:
%   cc: confusion matrix either in HITS, not percentage
%   flag_mean: if true, the average of all sensitivities and specificities will be returned (this is useful because the
%              cases when sensitivity or specificity is not defined (like rows or columns of cc containing all zeros) 
%              are discounted
%
% Outputs:
%   values: a no_classes X 2 matrix. First column are the sensitivities; second column are the specificities
%
% sens(i) = cc(i, i)/sum(cc(i, :))
% spec(i) = sum_{j, j ~= i} c(j, j) / sum_{j, j ~= i} sum(cc(j, :))
%
function values = calc_sens_spec(cc, flag_mean)


if ~exist('flag_mean')
    flag_mean = 0;
end;

no_classes = size(cc, 2);

values = zeros(no_classes, 2);

tr = trace(cc);
no_obs = sum(cc(:));

for i = 1:no_classes
    hits_class = cc(i, i);
    no_obs_class = sum(cc(i, :));
    values(i, 1)  = hits_class/no_obs_class;
    values(i, 2) = (tr-hits_class)/(no_obs-no_obs_class);
end;


if flag_mean
    values = values(:);
    values(isnan(values)) = [];
    values = mean(values(:));
end;

% Copyright 2010 Julio Trevisan, Plamen P. Angelov & Francis L. Martin.
% e-mailing author: j.trevisan@lancaster.ac.uk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% A copy of the GNU General Public License is shipped along with this
% program (filename: "COPYING").  For an online version of the license,
% see <http://www.gnu.org/licenses/>.
