% [S_B, S_W] = calculate_scatters(data, flag_modified_s_b = 0, penalty=0)
%              calculate_scatters(pieces, flag_modified_s_b = 0, penalty=0)
%
% Input:
%   data: structure with the following:
%     .X: dataset with rows as observations and columns as features;
%     .classes: column vector with corresponding classes (0-based)
%   flag_modified_s_b: if 1, S_B will be calculated in an alternative way
%     which is class terms will not be weighet by class sample size, which
%     will cause all classes to have equal importance.
%   P: penalty matrix to be added to S_W
%
% Outputs:
%  - S_W: "within-class scatter matrix"
%  - S_B: "inter-class scatter matrix"

function [S_B, S_W] = calculate_scatters(data, flag_modified_s_b, P)

[no, nf] = size(data.X);

if ~exist('flag_modified_s_b', 'var')
    flag_modified_s_b = 0;
end;

if ~exist('P', 'var')
    P = 0;
else
    [q, w] = size(P);
    if q ~= nf || w ~= nf
        % assumes the coefficients have been passed
        P = no*penalty_matrix(nf, P);
    else
    end;
end;

% First thing is split DATA according to CLASSES
%
% pieces = [o1, o2, ...]
% 
% o1.class = 0
% o1.data contains all DATA's rows from class 0
% .
% .
% . etc

pieces = data_split_classes(data);

m = mean(data.X); % total mean vector

no_classes = size(pieces, 2);


S_W = zeros(nf, nf);
S_B = S_W;
for i = 1:no_classes
    centered = pieces(i).X; % will contain the data with each centered to its mean.
    if size(centered, 1) > 0
        m_i = mean(centered); % vector containing the means for each column/feature.

        for j = 1:nf
            centered(:, j) = centered(:, j)-m_i(j); % centers each column to its mean
        end;

        n_i = size(centered, 1); % number of observations for class i

        pieces(i).scatter = centered'*centered; % scatter matrix. This is S_i in the reference

        if flag_modified_s_b
            S_B = S_B+(m_i-m)'*(m_i-m); % last term is a rank-1 matrix (MATLAB's default is row vector)
        else
            S_B = S_B+n_i*(m_i-m)'*(m_i-m); % last term is a rank-1 matrix (MATLAB's default is row vector)
        end;
        S_W = S_W+pieces(i).scatter;
    end;
end;

if 0
    S_B = S_B-P;
else
    % I have found empirically that adding P to S_W is equivalent to adding
    % P to X'*X in canonical correlation analysis

    S_W = S_W+P;
    
%     S_W = eye(nf, nf);
end;


% penalty is corrected (multiplied) by the number of samples because in the
% literature I saw penalty matrices summed together with covariance
% matrices, not scatter matrices.
% S_W = S_W+eye(nf, nf)*penalty*size(data.X, 1);
% Copyright 2010 Julio Trevisan, Plamen P. Angelov & Francis L. Martin.
% e-mailing author: j.trevisan@lancaster.ac.uk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% A copy of the GNU General Public License is shipped along with this
% program (filename: "COPYING").  For an online version of the license,
% see <http://www.gnu.org/licenses/>.
