% function data = data_line_correct(data, wns, no_convolutions)
%
% no_convolutions: number of convolutions from [1, 1, 1] in the average filter
%
% wns:
% IINITIAL = 1; IFINAL = 2; IFLAG0 = 3; IFLAGMIN = 4; INOCONV = 5
%
% IFLAG0 has prioriry over IFLAGMIN: if the former is 0, the latter is treated as 1
% if number of convolutions is 0, the average coefficients vector will be [1]

function data = data_line_correct(data, wns, no_convolutions)

NOCONV_DEFAULT = 4;

% Constants to access columns of 'wns'
IINITIAL = 1;
IFINAL = 2;
IFLAG0 = 3;
IFLAGMIN = 4;
INOCONV = 5;



[no_obs, no_fea] = size(data.X);


% converts wns to struct if needed
[nq, nw] = size(wns);
flag_wnisstruct = (nw == IFLAGMIN || nw == INOCONV) && sum(wns(1, IFLAG0) == [0 1]) > 0 && sum(wns(1, IFLAGMIN) == [0 1]) > 0;
if ~flag_wnisstruct
    % If single wavenumbers, not ranges, are in wns, defaults to wns to [wn, wn, flag_zero = 1, flag_min = 1; ...]
    if size(wns, 1) == 1; wns = wns'; end;
    no = length(wns);
    wns = [wns wns ones(no, 1) ones(no, 1)];     
end;

if ~exist('no_convolutions', 'var') && size(wns, 2) == 4
    % there is no convolution information
    no_convolutions = NOCONV_DEFAULT;
end;
    
if exist('no_convolutions', 'var')
    if length(no_convolutions) == 1
        no_convolutions = ones(size(wns, 1), 1)*no_convolutions;
    end;
    if size(no_convolutions, 2) > 1
        no_convolutions = no_convolutions';
    end;
    if size(wns, 2) == 4
        wns(1, 5) = 0; % Expands wns to 5th column
    end;
    wns(:, 5) = no_convolutions;
end;
        
    

% transforms wns in indexes. knots is parallel to first 2 columns of wns
knots = v_x2ind(wns(:, 1:2), data.x);
no_knots = size(knots, 1);


% minima = zeros(no_obs, no_knots);

% Filters
seed = [1, 1, 1];
filters = cell(1, no_knots);
filter_lengths = zeros(no_knots, 1); % number of points filter
filter_shifts = zeros(no_knots, 1); % shiftback: applied formula to simplify further
for iknot = 1:no_knots
    % creates an averaging filter
    filter = 1;
    for ii = 1:wns(iknot, INOCONV)
        filter = conv(filter, seed);
    end;
    filters{iknot} = filter/sum(filter);
    filter_lengths(iknot) = length(filter);
    filter_shifts(iknot) = (length(filter)-1)/2;


    if knots(iknot, 1)-filter_shifts(iknot) < 1
        error(sprintf('knot #%d: index %d is too close to beginning of spectrum!', iknot, knots(iknot, 1)));
    elseif knots(iknot, 2)+filter_shifts(iknot) > no_fea
        error(sprintf('knot #%d: index %d is too close to end of spectrum!', iknot, knots(iknot, 2)));
    end;
end;



% baseline correction
flag_record = 1;
if flag_record
    baselines = zeros(no_obs, no_fea);
end;

means = mean(data.X, 1); % row vector containing column means
for iobs = 1:no_obs
    if flag_record
        baseline = zeros(1, no_fea);
    end;
    
    for iknot = 1:no_knots
        idxshoriz = knots(iknot, 1):knots(iknot, 2);
        if wns(iknot, IFLAGMIN)
            [val, idxindeed] = min(data.X(iobs, idxshoriz));
            idxindeed = idxindeed+idxshoriz(1)-1;
%             minima(iobs, iknot) = idxindeed;
        else
            [val, idxindeed] = max(data.X(iobs, idxshoriz));
            idxindeed = idxindeed+idxshoriz(1)-1;
        end;
        
        % applies average filter to region centered in a minimum/maximum
        av = filters{iknot}*data.X(iobs, idxindeed-filter_shifts(iknot):idxindeed+filter_shifts(iknot))';

        if ~wns(iknot, IFLAG0) % ~IFLAG0 is average
            shifty = means(idxindeed);
        else
            shifty = 0;
        end;
        
        if iknot > 1

    
            p = polyfit([idxindeed_, idxindeed], [av_-shifty_, av-shifty], 1);
            
            if iknot == 2
               idx1 = 1;
            else
                idx1 = idxindeed_+1;
            end;
            if iknot == no_knots
                idx2 = no_fea;
            else
                idx2 = idxindeed;
            end;
          
          
            idxs = idx1:idx2;
            v = polyval(p, idxs);
            data.X(iobs, idxs) = data.X(iobs, idxs)-v;
            
            if flag_record
                baseline(idxs) = v;
            end;
        end;
        
        av_ = av;
        idxindeed_ = idxindeed;
        shifty_ = shifty;
        
    end;
    if flag_record
        baselines(iobs, :) = baseline;
%         figure; plot([means; data.X(iobs, :); baseline]'); legend({'means', 'X', 'baseline'});
%         input('vai nego');
%         disp('vai nego');
    end;
end;

if flag_record
    disp('hi, see the baselines');
end;
    
data = data_update_background(data);

if no_knots > 1
    fprintf('INFO: data has been baseline-corrected. Wns %s\n', mat2str(wns));
end;


% 
% minimax = minima;
% for i = 1:no_obs
%     minimax(i, :) = data.x(minima(i, :));
% end;
% 
% global dswheremin;
% dswheremin = data_set_X(data, minimax);
% 

% 
% 
% figure; 
% subplot(2, 1, 1);
% [N, X] = hist(minima(:, 1), length(knots(1, 1):knots(1, 2)));
% % bar(data.x(round(X)), N);
% bar(N);
% set(gca, 'Xdir', 'reverse');
% subplot(2, 1, 2);
% [N, X] = hist(minima(:, 2), length(knots(2, 1):knots(2, 2)));
% % bar(data.x(round(X)), N);
% bar(N);
% set(gca, 'Xdir', 'reverse');
% 
% keyboard;
% Copyright 2010 Julio Trevisan, Plamen P. Angelov & Francis L. Martin.
% e-mailing author: j.trevisan@lancaster.ac.uk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% A copy of the GNU General Public License is shipped along with this
% program (filename: "COPYING").  For an online version of the license,
% see <http://www.gnu.org/licenses/>.
