% data = data_new_from_txt(filename, range = [])
%
% Loads file and returns a struct containing the following elements:
%   data.X: sample dataset
%   data.classes: corresponding classes.
%   data.x: wavenumber vector, currently running from 1801.47 to 898.81
%   data.class_labels: {'0', '1', '2', ...
%
% Inputs:
%   filename: file name
%   range: initial and final values for the features labels. If not used, default will be [1801.47, 898.81] for the
%   standard file type or feature labels will be taken from the txt file in case of alternative file structure.
%
% Two file structures are supported:
%   - Table of data with no header and last column containing the class
%     index. Delimiters such as ';', ' ', or '\t' (Tab) are supported.
%
%   - Table with first row as x-axis values, first column as observation
%     identifiers and last column as class indexes. Note that the first and
%     last elements of the first row are not used. Only the ';'
%     delimiter is supported.
function data = data_new_from_txt(filename, range)

flag_range = exist('range', 'var'); 


data = data_new();
data.file_name = filename;
data.file_type = 'txt';


try
    A = load(filename);
    [rows, cols] = size(A);
    data = data_set_X(data, A(:, 1:cols-1));

    if ~flag_range
        data.x = linspace(1801.47, 898.81, cols-1); % TODO this is a hack!
    end;

    data.classes = A(:, cols);
catch me
    % If regular load fails, assumes the file has an alternative structure

    data = read_txt2(data, filename);
end;

if flag_range
    data.x = linspace(range(1), range(2), size(data.X, 2));
end;


% Assigns data.classes correcting numbers for eventual non-zero-based
% classes
m = min(data.classes);
if m > 0
    fprintf('WARNING: class numbers started in %d and thus were shifted down in order to start in zero.', m);
    data.classes = data.classes-m;
end;

% Produces class labels
for i = 1:max(data.classes)+1
    data.class_labels{i} = sprintf('Class %d', i-1);
end;

data = data_update_background(data);






%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
function data = read_txt2(data, filename)

% Opens for the first time to determine the number of columns
fid = fopen(filename);
s = fgets(fid);

% Tries to figure out which is the delimiter being used
delimiters = sprintf(';,\t');
no_delimiters = length(delimiters);
no_colss = zeros(1, no_delimiters);
for i = 1:no_delimiters
    no_colss(i) = sum(s == delimiters(i))+1;
end;    

[no_cols, index] = max(no_colss);
delimiter = delimiters(index);
fclose(fid);

% Mounts masks
mask_header = ['%q' repmat('%f', 1, no_cols-2) '%q'];

mask_data = ['%q' repmat('%f', 1, no_cols-2) '%d'];

% Opens for the second time in order to actually read the file
fid = fopen(filename);
c_header = textscan(fid, mask_header, 1, 'Delimiter', delimiter);
x = cell2mat(c_header(2:end-1));
trend = 0;
for i = 2:length(x)
  if trend == 0
      trend = sign(x(i)-x(i-1));
  elseif sign(x(i)-x(i-1)) ~= trend
      error('There is something wrong with this text file!');
  end;
end;



c_data = textscan(fid, mask_data, 'Delimiter', delimiter);
fclose(fid);

% Resolves values


data = data_set_X(data, cell2mat(c_data(2:end-1)));
data.x = x;
data.classes = double(cell2mat(c_data(end)));

labels = c_data{1}; 

% 'labels' may contain information more specific than needed.

no_obs = length(labels);
data.colony_codes = cell(no_obs, 1);
for i = 1:no_obs
    label = labels{i};
    idxs = regexp(label, '\.');
    
    if isempty(idxs)
        idx = length(label)+1;
    else
        idx = idxs(end); % The aim is to take everything before the last '.'
    end;
    
    data.colony_codes{i} = label(1:idx-1);
end;



% Copyright 2010 Julio Trevisan, Plamen P. Angelov & Francis L. Martin.
% e-mailing author: j.trevisan@lancaster.ac.uk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% A copy of the GNU General Public License is shipped along with this
% program (filename: "COPYING").  For an online version of the license,
% see <http://www.gnu.org/licenses/>.
