% output = data_normalize(X, params)
%          data_normalize(data, params)
%           data_normalize(p1, params, idxs_fea)
% 
%
% 
% data: data structure
% X: data matrix containing rows as observations, columns as features.
%
% Params: string containing one or more of the following:
%   a: normalizes to total area (makes area to be unity) (mutually exclusive 0).
%   n: 2-norm normalization per row (mutually exclusive 0).
%   v: full-range normalization across each row (if preset will be first task)
%      (mutually exclusive 0). 'idxs_fea' affects only this option
%   2: vector normalization across each row starting at wavenumber 1585
%      (will cause normalization to Amide II peak)
%      (mutually exclusive 0)
%   l: logit (log(absorp/(1-absorp))
%   c: centers X across columns (mutually exclusize 1)
%   s: centers X and forces variance to 1 across columns (mutually exclusize 1)
%   r: centers X and forces idxs_fea to 0 to 1 across columns (mutually exclusize 1) 
%
% idxs_fea: optional idxs_fea for vector normalization ('params'='v'). It is the FULL RANGE, NOT limits
function output = data_normalize(p1, params, idxs_fea)

flag_struct = 0;
if isstruct(p1)
    X = p1.X;
    flag_struct = 1;
else
    X = p1;
end;
[no, nf] = size(X);
if nargin() == 1
    params = '';
end;

flag_center = sum(params == 'c') > 0;
flag_var = sum(params == 's') > 0;
flag_range = sum(params == 'r') > 0;

if flag_var
    if sum(var(X) == 0)
        error('Can''t standardize data because there are variables with ZERO variance!');
    end;
end;
    
% HHH Horizontal normalization
flag_range_max = 0;
flag_area = sum(params == 'a') > 0;
flag_norm2 = sum(params == 'n') > 0;
flag_logit = sum(params == 'l') > 0;
if flag_area
    % Note: area calculation assumes the x-axis range to be unity, that's why 'a' below is divided by 'nf'
    % For ex., if all points are '1/nf', the total area will be 'nf'.
    for i = 1:no
        a = sum(X(i, :))/nf;
        X(i, :) = X(i, :)/a;
    end;
    
elseif flag_norm2
    for i = 1:no
        X(i, :) = X(i, :)/norm(X(i, :));
    end;
elseif flag_logit
    X = log(1./(1-X));
else
    [xx, yy] = meshgrid(params, 'v12');
    flag_range_max = sum(sum(xx == yy)) > 0;
    if flag_range_max
        if sum(params == 'v') > 0
            if ~exist('idxs_fea', 'var')
                idxs_fea = 1:size(X, 2);
            end;
        elseif sum(params == '1') > 0
            %***Attention: needs p1 as DATA
            % amide I peak
            idxs_fea = v_x2ind(1680, p1.x):v_x2ind(1610, p1.x);
        elseif sum(params == '2') > 0
            % amide II peak
            idxs_fea = v_x2ind(1570, p1.x):v_x2ind(1500, p1.x);
        end;
        for i = 1:no
            X(i, :) = X(i, :)/max(X(i, idxs_fea));
        end;
    end;
end;

% VVV Vertical normalization;
if flag_center || flag_var || flag_range
    means = mean(X, 1); % vector containing the means for each feature/column
    vars = var(X, 1); % variances of features/columns of X
    for icol = 1:nf
        colnowcentered = X(:, icol)-means(icol);

        if flag_center
            X(:, icol) = colnowcentered;
        elseif flag_var
            % forces variance to 1
            X(:, icol) = colnowcentered/sqrt(vars(icol));
        elseif flag_range
            % forces idxs_fea to 1
            mi = min(colnowcentered);
            ma = max(colnowcentered);
            X(:, icol) = colnowcentered/(ma-mi);
        end;
    end;
end;

if flag_struct
  p1.X = X;
  p1.type_norm = [p1.type_norm params];
  output = p1;
else
  output = X;
end;

if flag_area
    fprintf('INFO: data has been normalized to area 1\n');
end;

if flag_range_max
    fprintf('INFO: data has been horizontally idxs_fea-normalized range [%g, %g]\n', idxs_fea(1), idxs_fea(end));
end;
if flag_center
    disp('INFO: data has been vertically mean-centered.');
elseif flag_var
    disp('INFO: data has been vertically standardized.');
elseif flag_range
    disp('INFO: data has been vertically mean-centered and forced to be between [-0.5, +0.5].');
end;


% Copyright 2010 Julio Trevisan, Plamen P. Angelov & Francis L. Martin.
% e-mailing author: j.trevisan@lancaster.ac.uk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% A copy of the GNU General Public License is shipped along with this
% program (filename: "COPYING").  For an online version of the license,
% see <http://www.gnu.org/licenses/>.
