% plot_scatter_1d(data, idx_fea, flag_distr=1)
%
% Inputs:
%   data: data structure
%   idx_fea: what features to use. Numbers correpond to columns in data.X
%   flag_distr: whether to plot the distributions as well (facilitates visualization if classes are not very well separated)
function data_plot_scatter_1d(data, idx_fea, flag_distr)


if ~exist('flag_distr', 'var')
    flag_distr = 1;
end;

pieces = data_split_classes(data);
no_classes = size(pieces, 2);


% determines point range
xmin = +Inf;
xmax = -Inf;
for i = 1:no_classes
    mintemp = min(pieces(i).X(:, idx_fea(1)));
    maxtemp = max(pieces(i).X(:, idx_fea(1)));

    if mintemp < xmin
        xmin = mintemp;
    end;
    if maxtemp > xmax
        xmax = maxtemp;
    end;
end;


if flag_distr
    % Determines distribution multiplication factor. Classes with less points
    % will have a less high mountain. This gives a better visual feeling of the
    % outcome of a bayesian classifier.
    
    % I have dropped this because if there is a huge difference in number
    % of points between classes, the one with less points will have a
    % mountain which is too flat! Taking log() would make things more
    % presentable however wrong.
    
    if 0
        distrmult = zeros(1, no_classes);
        for i = 1:no_classes
            distrmult(i) = size(pieces(i).X, 1);
        end;

        if 0
            distrmult = log(distrmult);
        end;
        distrmult = distrmult/max(distrmult);
    else
        distrmult = ones(1, no_classes);
    end;
end;
    
extent = xmax-xmin;
k = 0.05;
xmin = xmin-extent*k;
xmax = xmax+extent*k;


hh = [];
for i = 1:no_classes
    y_offset = no_classes+1-i;
    x = pieces(i).X(:, idx_fea(1));
    no_x = size(x, 1);
    y = y_offset*ones(no_x, 1); % zero 1-column vector

    
    % Histogram
    if flag_distr
        [xdistr, ydistr] = distribution(x, 200);
        ydistr = distrmult(i)*ydistr/max(ydistr)*.75;
        plot(xdistr, ydistr+y_offset+.05, 'Color', [1, 1, 1]*.4, 'LineWidth', 2);
        hold on;
    end;

    
    hh(end+1) = plot(x, y, 'Color', find_color(i), 'Marker', find_marker(i), 'LineStyle', 'none', 'MarkerSize', 10, 'LineWidth', 2);
    hold on;
    plot(mean(x(:)), y(1), 'Color', find_color(i), 'Marker', find_marker(i), 'LineWidth', 3, 'MarkerSize', 15);
    
    
end


xmaxabs = max(abs([xmin, xmax]));
set(gca(), 'XLim', [-xmaxabs, xmaxabs]);

set(gca(), 'YTick', [])
set(gca(), 'YLim', [0 no_classes+1]);

feanames = data_get_feature_names(data, idx_fea);
xlabel(feanames{1});

legend(hh, data_get_legend(data));
title(sprintf('%s 1D Scatter Plot', data_get_title(data)));
format_frank();
% Copyright 2010 Julio Trevisan, Plamen P. Angelov & Francis L. Martin.
% e-mailing author: j.trevisan@lancaster.ac.uk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% A copy of the GNU General Public License is shipped along with this
% program (filename: "COPYING").  For an online version of the license,
% see <http://www.gnu.org/licenses/>.
