% [W_star]/[W_star, lambdas] = 
%   fisher_ld(data, flag_sphere=1, flag_modified_s_b=0, P=0)
% 
% Calculates Fisher's Linear Discriminant vectors (loadings):
%
% Inputs
%   data: structure with the following:
%     .X: dataset with rows as observations and columns as features;
%     .classes: column vector with corresponding classes (0-based)
%   flag_sphere = 0: forces S_W to be the identity.
%   flag_modified_s_b: see calculate_scatters()
%
% If either data or pieces is passed, the scatter matrixes will be
% calculated.
%
% If data is passed, the rank of S_W will be tested, and if it is lower
% than the dimensionality of the problem (i.e., number of features), PCA
% will be run first to project the data onto a space where the within-class 
% scatter matrix (say S_W_PCA) will be non-singular and then the obtained
% LDA loadings will be projected back to the original space.
%
% Fisher's Linear Discriminant according to R. O. Duda, Pattern
% Classification.
%
% The reference puts the equation to solve:
%
% S_B*w = lambda*S_W*w 
%
% There are rank(S_B) (at most no_classes-1) different w's.
%
%    [V,D] = EIG(A,B) produces a diagonal matrix D of generalized
%    eigenvalues and a full matrix V whose columns are the
%    corresponding eigenvectors so that A*V = B*V*D.
function [W_star, varargout] = fisher_ld(data, flag_sphere, flag_modified_s_b, P)

if ~exist('flag_modified_s_b', 'var')
    flag_modified_s_b = 0;
end;

s = 'None';
if ~exist('flag_sphere')
    flag_sphere = 1;
    s = 'Spheric covariance matrix';
end;

if ~exist('P')
    P = 0;
end;

flag_pca = 0;

[S_B, S_W] = calculate_scatters(data, flag_modified_s_b, P);
% S_W = diag(diag(S_W));

% S_W = eye(size(S_W, 1)); % TODO HACK let's try to see how it is dealt with if S_W is an eye

rsw = rank(S_W);
flag_deficient = rank(S_W) < length(S_W);

if flag_sphere && flag_deficient
  error('Cannot make S_W spherical because it is singular!');
end;

if flag_sphere
    [V, lambdas] = eig_ordered(S_W);
  
    V_ = V/sqrt(diag(lambdas));
    data.X = data.X*V_;

    [S_B, S_W] = calculate_scatters(data, flag_modified_s_b, P);
end;

if flag_deficient
    fprintf('Fisher''s LDA will be calculated in PC eigenspace because rank(S_W) = %d which is lower than %d\n', rsw, length(S_W));
    flag_pca = 1;

    [scores, loadings_pca] = princomp2(data.X);

    if rsw < size(scores, 2)
        fprintf('PCA Scores matrix will be truncated from %d to %d columns to match rank of S_W.\n', size(scores, 2), rsw);
    end;

    scores = scores(:, 1:rsw);
    loadings_pca = loadings_pca(:, 1:rsw);

    data = data_set_X(data, scores);

    [S_B, S_W] = calculate_scatters(data, flag_modified_s_b, P);
end;

no_classes = rank(S_B)+1;
no_features = size(S_B, 1);

rank_S_W = rank(S_W);
% if rank_S_W < no_features
%     error('Cannot calculate Fisher Linear Discriminant because rank(S_W) = %d which is lower than %d.\n', rank_S_W, no_features);
% end;

% if flag_sphere
% %     [V, D] = eigs(S_B, no_classes-1);
%      [V, D] = eig(S_B);
%      V = V(:, 1:(no_classes-1));
%      D = D(:, 1:(no_classes-1));
% else
%     OPTS.tol = 1e-50;
[V, D] = eigs(S_B, S_W, no_classes-1);
% end;
% [V, D] = eig(S_B, S_W);

no_lambdas = size(D, 2);

% Creates an index table and sorts it by eigenvalue in ascending order
% (because sortrows() hasn't a descending order option)
lambdas = diag(D);
% keyboard;
lambdas(:, 2) = (1:no_lambdas)';
lambdas = sortrows(lambdas);

% sweeps the [lambda, index] table backwards in search for no_classes-1
% largest and properly numeric eigenvalues
W_star = zeros(no_features, no_lambdas);
lambdas2 = zeros(no_lambdas, 1);
no_found = 0;
for i = no_lambdas:-1:1
    lambda = lambdas(i, 1);
    if lambda ~= NaN & lambda ~= Inf & lambda > 0
        no_found = no_found+1;
        
        v_temp = V(:, lambdas(i, 2));
        W_star(:, no_found) = v_temp/norm(v_temp);
        lambdas2(no_found) = lambdas(i, 1);
    end;
end;

if flag_sphere
    W_star = V_*W_star;
end;

if flag_pca
    W_star = loadings_pca*W_star;
end;

% Don't mess here again, JULIO, this needs to be like that, unnormalized
% W_star = adjust_loadings(W_star);


if nargout() == 2
  varargout(1) = {lambdas2}; 
end;

fprintf('INFO: F-LDA applied (pre-processing: ''%s'').\n', s);
% Copyright 2010 Julio Trevisan, Plamen P. Angelov & Francis L. Martin.
% e-mailing author: j.trevisan@lancaster.ac.uk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% A copy of the GNU General Public License is shipped along with this
% program (filename: "COPYING").  For an online version of the license,
% see <http://www.gnu.org/licenses/>.
