% function a = modelset_estimate(a, data)
%
% (A)
% All 2-class models will produce class estimations either 0 or 1.
% The original classes were however not necessarily 0/1 for
% no_classes*(no_classes-1)/2 models were created to handle a
% multi-class problem.
%
% The following lines thus convert the model's class estimations
% back to the original class numbers
%
% (B)
% Each model's estimation is added together. This is a pool. The
% winner class [for each observation] will be the one that receives
% more votes.

% For the pool we can use either each model's estimated class or
% its Y output. Using the Y output will take the 'degree of
% confidence' of the estimation into account whereas taking the
% classes will not reflect how sure each model is about its class
% estimation.
%
% In multiple-output models, however, flag_use_class is the only
% feasible option.
function a = modelset_estimate(a, data)

no_obs = size(data.X, 1);

if a.flag_preprocess
    data.X = data.X-repmat(a.preprocess_means, no_obs, 1);
    if a.preprocess_type == 's'
        data.X = data.X./repmat(a.preprocess_stds, no_obs, 1);
    end;
end;

if a.flag_transform
    data = data_transform_linear(data, a.transform_L); % Currently only lineaer transforms are supported
end;


% Here the modelset takes control of the number of classes
no_classes = length(a.class_labels);
scoretable = zeros(no_obs, no_classes);
if a.flag_split
    no_models = length(a.models);
% % % % % % % % %     entities = zeros(size(data.X, 1), 1);
   
    for i_model = 1:no_models
        [a.models{i_model}, a.o_ests(i_model)] = feval(a.models{i_model}.f_test, a.models{i_model}, data);
        

        % Each model only deals with 2 classes and has only 2 classes in
        % its .class_labels
        
        % Adds 1 somewhere at each line of the scoretable        
        est_classes = renumber_classes(a.o_ests(i_model).classes, a.o_ests(i_model).class_labels, a.class_labels);
        for iv = 1:no_obs
            idx = est_classes(iv)+1;
            scoretable(iv, idx) = scoretable(iv, idx)+1;
        end;
    end;
    
    a.o_est = est_new();
    a.o_est.class_labels = a.class_labels;
    [maxvotes, idxs] = max(scoretable, [], 2);
    a.o_est.classes = idxs-1;
    
else
    [a.models{1}, a.o_est] = feval(a.models{1}.f_test, a.models{1}, data);
end;
a.time_estimate = modelset_getbatch(a, 'time_estimate');  % This line is courtesy to make estimation time more shallowly accesible
% % % % a.o_est.rate = est_get_rate(a.o_est, data);


prop_names = a.models{1}.important_prop_names;
o = struct();
for i = 1:length(prop_names)
    name = prop_names{i};
    o.(name) = modelset_getbatch(a, name);
end;
a.important_props = o;

% Copyright 2010 Julio Trevisan, Plamen P. Angelov & Francis L. Martin.
% e-mailing author: j.trevisan@lancaster.ac.uk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% A copy of the GNU General Public License is shipped along with this
% program (filename: "COPYING").  For an online version of the license,
% see <http://www.gnu.org/licenses/>.
