% Principal Component Analysis (PCA)
%
% [scores] = princomp2(X, flag_rotate = 0)
% [scores, loadings] = princomp2(X, flag_rotate = 0)
% [scores, loadings, lambdas] = princomp2(X, flag_rotate = 0)
%
% Explanation:
%   PCA formula:
%     scores = X*loadings
%
%     Loadings are the eigenvectors of the X's scatter matrix.
%     The scatter matrix is defined ad X'*X, a simmetric positive
%     definite or semi-definite with rank r <= d
%
% Inputs:
%   X: [n][d] dataset matrix, columns stand for features and rows stand for
%      observations
%
% Outputs: 
%   scores: [n][r] PCA scores.
%   loadings: [d][r] loadings matrix (r is the rank of the dataset's
%             scatter matrix (see below for scatter matrix definition).
%             Loadings are actually the eigenvectors of the scatter matrix.
%   lambdas: [r][1] contains the eigenvalues of the scatter matrix.
%
% Note: the loadings vectors sometimes happen to point at the opposite directions
% of those obtaines by MATLAB's princomp() (not really a problem).
function [scores, varargout] = princomp2(X)

no_features = size(X, 2);

centered = data_normalize(X, 'c'); % center

cc = cov(centered); % Covariance matrix
r = rank(cc);

[vv, ll] = eig_ordered(cc); % eigenvectors, eigenvalues

vv = adjust_loadings(vv);

scores = centered*vv;
if nargout() == 2
  varargout(1) = {vv}; % loadings
elseif nargout() == 3
  varargout(1) = {vv}; % loadings
  
  
  varargout(2) = {ll};
end;

% Copyright 2010 Julio Trevisan, Plamen P. Angelov & Francis L. Martin.
% e-mailing author: j.trevisan@lancaster.ac.uk
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% A copy of the GNU General Public License is shipped along with this
% program (filename: "COPYING").  For an online version of the license,
% see <http://www.gnu.org/licenses/>.
